#include "../include/except.h"
#include "../include/mem.h"
#include <stdlib.h>

const Exception out_of_memory = { "Out Of Memory" }; // OOM Exception.

void *mem_alloc(ptrdiff_t nbytes, const char *file, int  line){
    void *ptr; // Memory to allocate.
    ASSERTED(nbytes > 0); 
    ptr = malloc(nbytes); // Attempt allocation.
    if (ptr == NULL){ // Out of memory, raise exception .
        if (file == NULL) // Wasn't called by macro.
            RAISE(out_of_memory);
        else // Called by macro.
            except_raise(&out_of_memory, file, line);
    }
    // Success, return allocated memory. 
    return ptr;
}
void *mem_calloc(ptrdiff_t count, ptrdiff_t nbytes,const char *file, int line) {
    void *ptr; // Memory to allocate.
    ASSERTED(count > 0);
    ASSERTED(nbytes > 0);
    ptr = calloc(count, nbytes);// Attempt allocation and zero out memory.
    if (ptr == NULL){ // Out of memory raise exception.
        if (file == NULL) // Wasn't called by macro.
            RAISE(out_of_memory);
        else // Was called by macro.
            except_raise(&out_of_memory, file, line);
    }
    // Success, return allocated and zeroed memory. 
    return ptr;
}
void mem_free(void *ptr, const char *file, int line) {
    (void) file;
    (void) line;
    if (ptr){// If we have valid pointer, free and null.
        free(ptr);
        ptr = NULL;
    }
}
void *mem_realloc(void *ptr, ptrdiff_t nbytes, const char *file, int line) {
    ASSERTED(ptr);
    ASSERTED(nbytes > 0);
    ptr = realloc(ptr, nbytes); // Attempt reallocation.
    if (ptr == NULL){ // Out of memory, raise exception.
        if (file == NULL) // Wasn't called by macro.
            RAISE(out_of_memory);
        else // Called by macro.
            except_raise(&out_of_memory, file, line);
    }
    // Success, return reallocated memory.
    return ptr;
}
